// Wraps a readahead peek function around any kind of token stream.

using System.Collections;



// what sort of things can be wrapped?
public interface IPeekable
{
	object Read();

	string Input { get; }
	int    Pos   { get; }
}



class TokenPeeker : IEnumerable
{
	// wrap an input source
	public TokenPeeker(IPeekable input)
	{
		mInput = input;

		mPos = input.Pos;
	}



	// look up the next token, without removing it from the input stream
	public object Peek()
	{
		if (mToken == null)
			mToken = mInput.Read();

		return mToken;
	}



	// read a token, removing it from the input stream
	public object Read()
	{
		object t = Peek();

		mToken = null;

		mPos = mInput.Pos;

		return t;
	}



	// make ourselves work with foreach, too
	public IEnumerator GetEnumerator()
	{
		while (Peek() != null)
			yield return Read();
	}



	// the input stream
	IPeekable mInput;



	// the current token, or null if we don't have one
	object mToken;



	// current read position
	int mPos;



	// accessors
	public string Input
	{
		get { return mInput.Input; }
	}

	public int Pos
	{
		get { return mPos; }
	}
}


