// Complex number type.

using System;
using System.Globalization;



struct Complex
{
	public double Real;
	public double Imag;



	// construct from real and imaginary parts
	public Complex(double real, double imag)
	{
		Real = real;
		Imag = imag;
	}



	// automatic conversion from real to complex
	public static implicit operator Complex(double real)
	{
		return new Complex(real, 0);
	}



	// addition operator
	public static Complex operator + (Complex a, Complex b)
	{
		return new Complex(a.Real + b.Real,
						   a.Imag + b.Imag);
	}



	// unary negation operator	
	public static Complex operator - (Complex a)
	{
		return new Complex(-a.Real,
						   -a.Imag);
	}



	// subtraction operator
	public static Complex operator - (Complex a, Complex b)
	{
		return new Complex(a.Real - b.Real,
						   a.Imag - b.Imag);
	}



	// multiplication operator
	public static Complex operator * (Complex a, Complex b)
	{
		if ((a.Imag == 0) && (b.Imag == 0))
			return a.Real * b.Real;

		return new Complex(a.Real * b.Real - a.Imag * b.Imag,
						   a.Real * b.Imag + a.Imag * b.Real);
	}



	// division operator
	public static Complex operator / (Complex a, Complex b)
	{
		if ((a.Imag == 0) && (b.Imag == 0))
			return a.Real / b.Real;

		double r = a.Real * b.Real + a.Imag * b.Imag;
		double i = a.Imag * b.Real - a.Real * b.Imag;

		double d = b.Real * b.Real + b.Imag * b.Imag;

		return new Complex(r/d, i/d);
	}



	// measure our magnitude
	public double Abs()
	{
		return Math.Sqrt(Real*Real + Imag*Imag);
	}



	// callback for converting a double to a string
	public delegate string FormatHandler(double value);



	// convert to string, using a custom number format
	public string ToString(FormatHandler formatter)
	{
		if (Imag == 0)
			return formatter(Real);

		string i = (Imag == 1) ? "i" : formatter(Imag) + "i";

		if (Real == 0)
			return i;

		return formatter(Real) + "+" + i;
	}



	// convert to string, using the default number format
	public override string ToString()
	{
		return ToString(delegate(double value) { return value.ToString(CultureInfo.InvariantCulture); } );
	}
}
