// Numeric range type.

using System;
using System.Collections.Generic;



class Range
{
	// default constructor
	public Range()
	{
		mMin = -1;
		mMax = 1;
	}



	// explicit constructor
	public Range(double min, double max)
	{
		mMin = min;
		mMax = max;
	}



	// range extents
	double mMin;
	double mMax;



	// are we allowed to grow?
	bool mLocked;

	public void Lock()
	{
		mLocked = true;
	}



	// expand the range
	public void Add(Range range)
	{
		Add(range.mMin);
		Add(range.mMax);
	}



	// expand the range
	public void Add(double value)
	{
		if (mLocked)
			throw new InvalidOperationException("Can't add to a locked range.");

		mMin = Math.Min(mMin, value);
		mMax = Math.Max(mMax, value);
	}



	// accessors
	public double Min
	{
		get { return mMin; }
	}

	public double Max
	{
		get { return mMax; }
	}

	public double Size
	{
		get { return mMax - mMin; }
	}

	public bool Locked
	{
		get { return mLocked; }
	}



	// enumerate by dividing the range into a fixed number of steps
	public IEnumerable<double> Enumerate(int steps)
	{
		return new Enumerator(this, steps);
	}



	// iterate over a subdivision of the range
	class Enumerator : IEnumerable<double>
	{
		public Enumerator(Range range, int steps)
		{
			mRange = range;
			mSteps = steps;
		}


		public IEnumerator<double> GetEnumerator()
		{
			double pos = mRange.Min;
			double step = mRange.Size / mSteps;

			for (int n=0; n<mSteps; n++) {
				yield return pos;
				pos += step;
			}
		}


		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return GetEnumerator();
		}


		Range mRange;
		int mSteps;
	}
}
